@@ -24,6 +24,7 @@
 #include <gsdefaults.h>
 #include <ggvutils.h>
 #include <ggv-msg-window.h>
+#include <ggv-file-sel.h>
 #include <ggv-prefs.h>
 #include <ggv-prefs-ui.h>
 #include <ggv-sidebar.h>
@@ -31,6 +32,8 @@
 
 #include <bonobo.h>
 
+#include <libgnomevfs/gnome-vfs.h>
+
 struct _GgvPostScriptViewPrivate {
 	GtkWidget *gs;
 
@@ -40,20 +43,20 @@ struct _GgvPostScriptViewPrivate {
 	gint magstep;
 	gboolean pan;
 	gdouble prev_x, prev_y;
-	gchar *tmp_name;
+	gchar *tmp_name, *uri, *save_path;
 	guint gconf_notify_id;
 
 	GgvMsgWindow *msg_win;
 	GgvSidebar *sidebar;
 
-	GtkWidget *save_file_sel;
-
 	BonoboPropertyBag     *property_bag;
 	BonoboPropertyControl *property_control;
 
 	BonoboUIComponent     *uic, *popup_uic;
 
 	gboolean               zoom_fit;
+
+	gboolean pane_auto_jump, page_flip;
 };
 
 struct _GgvPostScriptViewClassPrivate {
@@ -87,7 +90,7 @@ static GdkCursor *pan_cursor = NULL;
 static BonoboObjectClass *ggv_postscript_view_parent_class;
 
 static const gchar *orientation_paths[] = {
-	"/commands/OverrideOrientation",
+	"/commands/OrientationDocument",
 	"/commands/OrientationPortrait",
 	"/commands/OrientationLandscape",
 	"/commands/OrientationUpsideDown",
@@ -96,7 +99,7 @@ static const gchar *orientation_paths[] = {
 };
 
 static const gchar *size_paths[] = {
-	"/commands/OverrideSize",
+	"/commands/PaperSizeDoc",
 	"/commands/PaperSizeBBox",
 	"/commands/PaperSizeLetter",
 	"/commands/PaperSizeTabloid",
@@ -104,6 +107,9 @@ static const gchar *size_paths[] = {
 	"/commands/PaperSizeLegal",
 	"/commands/PaperSizeStatement",
 	"/commands/PaperSizeExecutive",
+	"/commands/PaperSizeA0",
+	"/commands/PaperSizeA1",
+	"/commands/PaperSizeA2",
 	"/commands/PaperSizeA3",
 	"/commands/PaperSizeA4",
 	"/commands/PaperSizeA5",
@@ -118,11 +124,11 @@ static const gchar *size_paths[] = {
 void
 get_status(GgvPostScriptView *ps_view, BonoboArg *arg)
 {
-	gchar *status;
+	const gchar *status;
 
 	g_assert(arg->_type == TC_CORBA_string);
 	
-	status = "GGV control status...";
+	status = GTK_GS(ps_view->priv->gs)->gs_status;;
 
 	*(CORBA_string *)arg->_value = CORBA_string_dup(status);
 }
@@ -148,19 +154,23 @@ get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 static void
 set_file_items_sensitivity(GgvPostScriptView *ps_view, gboolean sens)
 {
-	gchar *prop_val = sens?"1":"0";
+	gchar *prop_val;
 	Bonobo_UIContainer container;
 
 	container = bonobo_ui_component_get_container (ps_view->priv->uic);
 	if (container == CORBA_OBJECT_NIL)
 		return;
 
+	prop_val = sens?"1":"0";
+
+	bonobo_ui_component_freeze(ps_view->priv->uic, NULL);
 	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FileSaveMarked",
 								 "sensitive", prop_val, NULL);
 	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FilePrintMarked",
 								 "sensitive", prop_val, NULL);
 	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FilePrintAll",
 								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_thaw(ps_view->priv->uic, NULL);
 }
 
 static void
@@ -174,6 +184,7 @@ _set_page_items_sensitivity(GgvPostScriptView *ps_view, BonoboUIComponent *uic)
 	if (container == CORBA_OBJECT_NIL)
 		return;
 
+	bonobo_ui_component_freeze(uic, NULL);
 	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1 ||
 				gtk_gs_get_current_page(gs) < 0)?
 		"0":"1";
@@ -190,6 +201,7 @@ _set_page_items_sensitivity(GgvPostScriptView *ps_view, BonoboUIComponent *uic)
 		"0":"1";
 	bonobo_ui_component_set_prop(uic, "/commands/LastPage",
 								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_thaw(uic, NULL);
 }
 
 static void
@@ -207,6 +219,8 @@ notify_page_count_change(GgvPostScriptView *ps_view)
 
 	set_page_items_sensitivity(ps_view);
 	set_file_items_sensitivity(ps_view, ggv_postscript_view_get_page_count(ps_view) > 0);
+	if(ps_view->priv->sidebar)
+		ggv_sidebar_create_page_list(ps_view->priv->sidebar);
 
 	arg = bonobo_arg_new(TC_CORBA_long);
 	BONOBO_ARG_SET_LONG(arg, ggv_postscript_view_get_page_count(ps_view));
@@ -259,6 +273,18 @@ notify_title_change(GgvPostScriptView *ps_view)
 	bonobo_arg_release(arg);
 }
 
+static void
+notify_status_change(GgvPostScriptView *ps_view)
+{
+	BonoboArg *arg;
+	arg = bonobo_arg_new(TC_CORBA_string);
+	get_status(ps_view, arg);
+	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+										 "Bonobo/Property:change:status",
+										 arg, NULL);
+	bonobo_arg_release(arg);
+}
+
 void
 ggv_postscript_view_goto_page(GgvPostScriptView *ps_view, gint page)
 {
@@ -267,6 +293,10 @@ ggv_postscript_view_goto_page(GgvPostScriptView *ps_view, gint page)
 	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs), page);
 	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) != old_page) {
 		notify_page_change(ps_view);
+		if(ps_view->priv->pane_auto_jump) {
+			gtk_gs_scroll_to_edge(GTK_GS(ps_view->priv->gs),
+								  GTK_POS_TOP,  GTK_POS_TOP);
+		}
 	}
 }
 
@@ -286,7 +316,6 @@ static void
 load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 {
 	gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
-	ggv_sidebar_create_page_list(ps_view->priv->sidebar);
 	notify_title_change(ps_view);
 	notify_page_count_change(ps_view);
 	notify_orientation_change(ps_view);
@@ -294,6 +323,39 @@ load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 	notify_page_change(ps_view);
 	ggv_sidebar_update_coordinates(ps_view->priv->sidebar, 0.0, 0.0);
 	set_file_items_sensitivity(ps_view, ggv_postscript_view_get_page_count(ps_view) > 0);
+	notify_status_change(ps_view);
+}
+
+static void
+ps_view_clean_tmp_file(GgvPostScriptView *ps_view)
+{
+	/* copy stream to a tmp file */
+	if(ps_view->priv->tmp_name != NULL) {
+		unlink(ps_view->priv->tmp_name);
+		g_free(ps_view->priv->tmp_name);
+		ps_view->priv->tmp_name = NULL;
+	}
+}
+
+static FILE *
+ps_view_get_tmp_file(GgvPostScriptView *ps_view)
+{
+	int fd;
+	FILE *tmpfile;
+
+	ps_view_clean_tmp_file(ps_view);
+	ps_view->priv->tmp_name = g_strconcat(g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+	if((fd = mkstemp(ps_view->priv->tmp_name)) < 0) {
+		g_free(ps_view->priv->tmp_name),
+		ps_view->priv->tmp_name = NULL;
+		return NULL;
+	}
+	tmpfile = fdopen(fd, "w");
+	if(!tmpfile) {
+		close(fd);
+		return NULL;
+	}
+	return tmpfile;
 }
 
 /*
@@ -317,23 +379,8 @@ load_ps_from_stream (BonoboPersistStream *ps,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
-	/* copy stream to a tmp file */
-	if(ps_view->priv->tmp_name != NULL) {
-		unlink(ps_view->priv->tmp_name);
-		g_free(ps_view->priv->tmp_name);
-		ps_view->priv->tmp_name = NULL;
-	}
-	ps_view->priv->tmp_name = g_strconcat(g_get_tmp_dir(), "/ggvXXXXXX", NULL);
-	if((fd = mkstemp(ps_view->priv->tmp_name)) < 0) {
-		g_free(ps_view->priv->tmp_name),
-		ps_view->priv->tmp_name = NULL;
-		return;
-	}
-	tmpfile = fdopen(fd, "w");
-	if(!tmpfile) {
-		close(fd);
-		return;
-	}
+	ps_view_clean_tmp_file(ps_view);
+	tmpfile = ps_view_get_tmp_file(ps_view);
 
 	do {
 		Bonobo_Stream_read (stream, 32768, &buffer, ev);
@@ -465,14 +512,65 @@ static gint
 load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 				   CORBA_Environment *ev, void *data)
 {
+	GnomeVFSResult result;
+	GnomeVFSHandle *handle;
+	GnomeVFSURI     *uri;
+	GnomeVFSFileSize bytes_read = 0;
+	guchar *buffer;
 	GgvPostScriptView *ps_view;
+	gchar *local_file = NULL, *uri_str;
+	FILE *tmpfile;
 
 	g_return_val_if_fail (data != NULL, -1);
 	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (data), -1);
 
-	ps_view = GGV_POSTSCRIPT_VIEW (data);
+	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
-	load_ps(ps_view, filename);
+	ps_view_clean_tmp_file(ps_view);
+
+	uri = gnome_vfs_uri_new(filename);
+	if(gnome_vfs_uri_is_local(uri)) {
+		uri_str = gnome_vfs_uri_to_string(uri, GNOME_VFS_URI_HIDE_NONE);
+		local_file = gnome_vfs_get_local_path_from_uri(uri_str);
+		g_free(uri_str);
+	}
+	else {
+		/* open uri */
+		result = gnome_vfs_open_uri(&handle, uri, GNOME_VFS_OPEN_READ);
+		gnome_vfs_uri_unref(uri);
+		if(result != GNOME_VFS_OK) {
+			return -1;
+		}
+
+		tmpfile = ps_view_get_tmp_file(ps_view);
+
+		buffer = g_new0(guchar, 4096);
+		do {
+			result = gnome_vfs_read (handle, buffer,
+									 4096, &bytes_read);
+			if (result != GNOME_VFS_OK)
+				break;
+			
+			if (bytes_read > 0)
+				if(fwrite(buffer, 1, bytes_read, tmpfile) != bytes_read) {
+					result = GNOME_VFS_ERROR_GENERIC;
+					break;
+				}
+		} while(TRUE);
+
+		gnome_vfs_close(handle);
+		fclose(tmpfile);
+
+		if(result != GNOME_VFS_ERROR_EOF)
+			return -1;
+	}
+	if(ps_view->priv->tmp_name) {
+		load_ps(ps_view, ps_view->priv->tmp_name);
+	}
+	else if(local_file) {
+		load_ps(ps_view, local_file);
+		g_free(local_file);
+	}
 
 	return 0;
 }
@@ -482,9 +580,9 @@ listener_Orientation_cb (BonoboUIComponent *uic, const char *path,
 						 Bonobo_UIComponent_EventType type, const char *state,
 						 gpointer user_data)
 {
-	BonoboArg *arg;
 	GgvPostScriptView *ps_view;
-	GNOME_GGV_Orientation orientation;
+	GtkGSOrientation orientation;
+	GtkGS *gs;
 
 	g_return_if_fail(user_data != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW (user_data));
@@ -496,84 +594,74 @@ listener_Orientation_cb (BonoboUIComponent *uic, const char *path,
 		return;
 
 	ps_view = GGV_POSTSCRIPT_VIEW(user_data);
+	gs = GTK_GS(ps_view->priv->gs);
 
-	if(!strcmp(path, "OrientationPortrait"))
-		orientation = GNOME_GGV_ORIENTATION_PORTRAIT;
-	else if(!strcmp(path, "OrientationLandscape"))
-		orientation = GNOME_GGV_ORIENTATION_LANDSCAPE;
-	else if(!strcmp(path, "OrientationUpsideDown"))
-		orientation = GNOME_GGV_ORIENTATION_UPSIDEDOWN;
-	else if(!strcmp(path, "OrientationSeascape"))
-		orientation = GNOME_GGV_ORIENTATION_SEASCAPE;
+	if(!strcmp(path, "OrientationDocument")) {
+		if(gs->override_orientation == TRUE) {
+			gtk_gs_set_override_orientation(GTK_GS(ps_view->priv->gs), FALSE);
+			notify_orientation_change(ps_view);
+		}
+	}
 	else {
-		g_warning("Unknown orientation `%s'", path);
-		return;
+		if(!strcmp(path, "OrientationPortrait"))
+			orientation = GTK_GS_ORIENTATION_PORTRAIT;
+		else if(!strcmp(path, "OrientationLandscape"))
+			orientation = GTK_GS_ORIENTATION_LANDSCAPE;
+		else if(!strcmp(path, "OrientationUpsideDown"))
+			orientation = GTK_GS_ORIENTATION_UPSIDEDOWN;
+		else if(!strcmp(path, "OrientationSeascape"))
+			orientation = GTK_GS_ORIENTATION_SEASCAPE;
+		else {
+			g_warning("Unknown orientation `%s'", path);
+			return;
+		}
+		gtk_gs_set_default_orientation(GTK_GS(ps_view->priv->gs), orientation);
+		if(gs->override_orientation == FALSE) {
+			gtk_gs_set_override_orientation(GTK_GS(ps_view->priv->gs), TRUE);
+		}
+		notify_orientation_change(ps_view);		
 	}
-
-	arg = bonobo_arg_new(TC_GNOME_GGV_Orientation);
-	BONOBO_ARG_SET_GENERAL(arg, orientation, TC_GNOME_GGV_Orientation,
-						   GNOME_GGV_Orientation, NULL);
-
-	bonobo_pbclient_set_value(BONOBO_OBJREF(ps_view->priv->property_bag),
-							  "default_orientation", arg, NULL);
-
-	bonobo_arg_release(arg);
 }
 
 static void
-listener_Toggle_cb (BonoboUIComponent *uic, const char *path,
-					Bonobo_UIComponent_EventType type, const char *state,
-					gpointer user_data)
+sync_orientation_items(GgvPostScriptView *ps_view)
 {
-	BonoboArg *arg;
-	GgvPostScriptView *ps_view;
-	CORBA_boolean flag;
-	gchar *prop;
-	int i;
-
-	static gchar *properties[][2] = {
-		{ "OverrideOrientation", "override_orientation" },
-		{ "OverrideSize", "override_size" },
-		{ "RespectEOF", "respect_eof" },
-		{ "Antialiasing", "antialiasing" },
-		{ "WatchFile", "watch_file" },
-		{ NULL, NULL }
-	};
-
-	g_return_if_fail(user_data != NULL);
-	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW (user_data));
-
-	if(type != Bonobo_UIComponent_STATE_CHANGED)
-		return;
-
-	if(!state)
-		return;
+	gint i;
+	gboolean orient_state;
 	
-	ps_view = GGV_POSTSCRIPT_VIEW(user_data);
-
-	if(atoi(state))
-		flag = CORBA_TRUE;
-	else
-		flag = CORBA_FALSE;
-
-	prop = NULL;
-	for(i = 0; properties[i][0]; i++) {
-		if(!strcmp(path, properties[i][0])) {
-			prop = properties[i][1];
-			break;
-		}
+	bonobo_ui_component_freeze(ps_view->priv->uic, NULL);
+	for(i = 0; orientation_paths[i] != NULL; i++) {
+		if(i > 0)
+			orient_state = ((TRUE == GTK_GS(ps_view->priv->gs)->override_orientation) &&
+							(i - 1 == gtk_gs_get_default_orientation(GTK_GS(ps_view->priv->gs))));
+		else
+			orient_state = (FALSE == GTK_GS(ps_view->priv->gs)->override_orientation);
+		bonobo_ui_component_set_prop(ps_view->priv->uic, orientation_paths[i],
+									 "state", orient_state?"1":"0", NULL);
 	}
+	bonobo_ui_component_thaw(ps_view->priv->uic, NULL);
+}
 
-	g_assert(prop != NULL);
+static void
+sync_size_items(GgvPostScriptView *ps_view)
+{
+	gint i;
+	gboolean size_state;
+	const gchar **paths;
 
-	arg = bonobo_arg_new(TC_CORBA_boolean);
-	BONOBO_ARG_SET_GENERAL(arg, flag, TC_CORBA_boolean,
-						   CORBA_boolean, NULL);
+	paths = size_paths;
 
-	bonobo_pbclient_set_value(BONOBO_OBJREF(ps_view->priv->property_bag),
-							  prop, arg, NULL);
-
-	bonobo_arg_release(arg);
+	bonobo_ui_component_freeze(ps_view->priv->uic, NULL);
+	for(i = 0; paths[i] != NULL; i++) {
+		if(i > 0)
+			size_state = ((TRUE == GTK_GS(ps_view->priv->gs)->override_size) &&
+						  (i - 1 == gtk_gs_get_default_size(GTK_GS(ps_view->priv->gs))));
+		else
+			size_state = (FALSE == GTK_GS(ps_view->priv->gs)->override_size);
+		bonobo_ui_component_set_prop(ps_view->priv->uic, paths[i],
+									 "state", size_state?"1":"0", NULL);
+	}
+	bonobo_ui_component_thaw(ps_view->priv->uic, NULL);
 }
 
 static void
@@ -582,8 +670,8 @@ listener_Size_cb(BonoboUIComponent *uic, const char *path,
 				  gpointer user_data)
 {
 	GgvPostScriptView *ps_view;
-	GNOME_GGV_Size size;
-	BonoboArg *arg;
+	gint size;
+	GtkGS *gs;
 
 	g_return_if_fail(user_data != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(user_data));
@@ -595,17 +683,23 @@ listener_Size_cb(BonoboUIComponent *uic, const char *path,
 		return;
 
 	ps_view = GGV_POSTSCRIPT_VIEW(user_data);
+	gs = GTK_GS(ps_view->priv->gs);
 
-	size = CORBA_string_dup(path + strlen("PaperSize"));
-
-	arg = bonobo_arg_new(TC_GNOME_GGV_Size);
-	BONOBO_ARG_SET_GENERAL(arg, size, TC_GNOME_GGV_Size,
-						   GNOME_GGV_Size, NULL);
-
-	bonobo_pbclient_set_value(BONOBO_OBJREF(ps_view->priv->property_bag),
-							  "default_size", arg, NULL);
-
-	bonobo_arg_release(arg);
+	if(!strcmp(path, "PaperSizeDoc")) {
+		if(gs->override_size == TRUE) {
+			gtk_gs_set_override_size(GTK_GS(ps_view->priv->gs), FALSE);
+			sync_size_items(ps_view);
+		}
+	}
+	else {
+		size = gtk_gs_get_size_index(path + strlen("PaperSize"),
+									 gtk_gs_defaults_get_paper_sizes());
+		gtk_gs_set_default_size(GTK_GS(ps_view->priv->gs), size);
+		if(gs->override_size == FALSE) {
+			gtk_gs_set_override_size(GTK_GS(ps_view->priv->gs), TRUE);
+		}
+		sync_size_items(ps_view);
+	}
 }
 
 
@@ -662,74 +756,30 @@ ggv_postscript_view_print(GgvPostScriptView *ps_view, gchar *ps)
 	g_strfreev(argv);
 }
 
-static void
-save_file_sel_unmap(GtkWidget *widget)
-{
-	ggv_get_window_size(widget, &file_sel_width, &file_sel_height);
-}
-
 static gboolean
-save_file_sel_delete_event(GtkWidget *widget, GdkEventAny *e, gpointer data)
+ps_view_save_doc(GgvPostScriptView *ps_view, const gchar *fname,
+				 const gchar *doc, gint len)
 {
-	gtk_widget_hide(widget);
-	return TRUE;
-}
+	GnomeVFSResult result;
+	GnomeVFSHandle *handle;
+	GnomeVFSURI *uri;
+	GnomeVFSFileSize written;
 
-static void
-save_file_sel_ok_clicked(GtkWidget *widget, gpointer data)
-{
-	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
-	gint *active_rows;
-	const gchar *fname;
-	gchar *doc;
-	FILE *f;
-
-	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
-	doc = ggv_postscript_view_get_ps(ps_view, active_rows);
-	g_free(active_rows);
-
-	/* get file name */
-	gtk_widget_hide(ps_view->priv->save_file_sel);
-
-	if(doc != NULL) {
-		fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel));
-
-		if(ggv_file_readable(fname)) {
-			GtkWidget *dlg;
-		
-			dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
-										 GTK_DIALOG_MODAL,
-										 GTK_MESSAGE_QUESTION,
-										 GTK_BUTTONS_YES_NO,
-										 _("A file with this name already exists.\n"
-										   "Do you want to overwrite it?"));
-			gtk_widget_show(dlg);
-			switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
-			case GTK_RESPONSE_NO:
-			case GTK_RESPONSE_DELETE_EVENT:
-				gtk_widget_destroy(dlg);
-				g_free(doc);
-				return;
-			default:
-				break;
-			}
-			gtk_widget_destroy(dlg);
-		}
-		
-		if(fname && (f = fopen(fname, "w")) != NULL) {
-			fwrite(doc, 1, strlen(doc), f);
-			fclose(f);
-		}
-		g_free(doc);
+	uri = gnome_vfs_uri_new (fname);
+	result = gnome_vfs_create_uri (&handle, uri,
+								   GNOME_VFS_OPEN_WRITE, FALSE, 0644);
+	gnome_vfs_uri_unref(uri);
+	if (result != GNOME_VFS_OK) {
+		return FALSE;
 	}
-}
 
-static void
-save_file_sel_cancel_clicked(GtkWidget *widget, gpointer data)
-{
-	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+	result = gnome_vfs_write(handle, doc, len, &written);
+	gnome_vfs_close(handle);
 
-	gtk_widget_hide(ps_view->priv->save_file_sel);
+	if(result != GNOME_VFS_OK || written != len)
+		return FALSE;
+	else
+		return TRUE;
 }
 
 static void
@@ -737,7 +787,8 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 	gint *active_rows;
-	
+	gchar *doc, *fname, *local_path = NULL;
+
 	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
 	if(active_rows[0] == -1) {
 		GtkWidget *dlg;
@@ -761,37 +812,44 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 	}
 	g_free(active_rows);
 	
-	if(!ps_view->priv->save_file_sel) {
-		gchar *home_path = g_strconcat(g_get_home_dir(), "/", NULL);
-		ps_view->priv->save_file_sel = gtk_file_selection_new(_("Select a file to save pages as"));
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel),
-										home_path);
-		g_free(home_path);
-		gtk_window_set_transient_for(GTK_WINDOW(ps_view->priv->save_file_sel),
-									 GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)));
-		gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(ps_view->priv->save_file_sel));
-		gtk_window_set_resizable(GTK_WINDOW(ps_view->priv->save_file_sel), TRUE);
-		g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(ps_view->priv->save_file_sel)->ok_button),
-						 "clicked", G_CALLBACK(save_file_sel_ok_clicked), ps_view);
-		g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(ps_view->priv->save_file_sel)->cancel_button),
-						 "clicked", G_CALLBACK(save_file_sel_cancel_clicked), ps_view);
-		g_signal_connect(G_OBJECT(ps_view->priv->save_file_sel), "delete_event",
-						 G_CALLBACK(save_file_sel_delete_event), ps_view);
-		g_signal_connect(G_OBJECT(ps_view->priv->save_file_sel), "unmap",
-						 G_CALLBACK(save_file_sel_unmap), NULL);
-	}
-	if(!GTK_WIDGET_VISIBLE(ps_view->priv->save_file_sel)) {
-		gtk_widget_set_usize(GTK_WIDGET(ps_view->priv->save_file_sel),
-							 file_sel_width, file_sel_height);
-		if(GTK_GS(ps_view->priv->gs)->gs_filename) {
-			gchar *path = g_path_get_dirname(GTK_GS(ps_view->priv->gs)->gs_filename);
-			gchar *path2 = g_strconcat(path, "/", NULL);
-			g_free(path);
-			gtk_file_selection_set_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel), path2);
-			g_free(path2);
+	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
+	doc = ggv_postscript_view_get_ps(ps_view, active_rows);
+	g_free(active_rows);
+
+	if(doc != NULL) {
+		fname = ggv_file_sel_request_uri(_("Select a file to save pages as"),
+										 ps_view->priv->save_path);
+		if(fname) {
+			if((local_path = gnome_vfs_get_local_path_from_uri(fname)) &&
+			   ggv_file_length(local_path) >= 0) {
+				GtkWidget *dlg;
+		
+				g_free(local_path);
+				dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
+											 GTK_DIALOG_MODAL,
+											 GTK_MESSAGE_QUESTION,
+											 GTK_BUTTONS_YES_NO,
+											 _("A file with this name already exists.\n"
+											   "Do you want to overwrite it?"));
+				gtk_widget_show(dlg);
+				switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
+				case GTK_RESPONSE_NO:
+				case GTK_RESPONSE_DELETE_EVENT:
+					gtk_widget_destroy(dlg);
+					g_free(fname);
+					g_free(doc);
+					return;
+				default:
+					break;
+				}
+				gtk_widget_destroy(dlg);
+			}
+			ps_view_save_doc(ps_view, fname, doc, strlen(doc));
+			if(ps_view->priv->save_path)
+				g_free(ps_view->priv->save_path);
+			ps_view->priv->save_path = fname;
 		}
-		gtk_widget_show(ps_view->priv->save_file_sel);
-		ggv_raise_and_focus_widget(ps_view->priv->save_file_sel);
+		g_free(doc);
 	}
 }
 
@@ -914,45 +972,6 @@ verb_SettingsPreferences(BonoboUIComponent *uic, gpointer data, const char *cnam
 		ggv_raise_and_focus_widget(dlg);
 }
 
-static void
-sync_orientation_items(GgvPostScriptView *ps_view)
-{
-	gint i;
-	gboolean val, orient_state;
-
-	val = gtk_gs_get_override_orientation(GTK_GS(ps_view->priv->gs));
-	bonobo_ui_component_set_prop(ps_view->priv->uic, orientation_paths[0],
-								 "state", val?"1":"0", NULL);
-	for(i = 1; orientation_paths[i] != NULL; i++) {
-		bonobo_ui_component_set_prop(ps_view->priv->uic, orientation_paths[i],
-									 "sensitive", val?"1":"0", NULL);
-		orient_state = (i - 1 == gtk_gs_get_default_orientation(GTK_GS(ps_view->priv->gs)));
-		bonobo_ui_component_set_prop(ps_view->priv->uic, orientation_paths[i],
-									 "state", orient_state?"1":"0", NULL);
-	}
-}
-
-static void
-sync_size_items(GgvPostScriptView *ps_view)
-{
-	gint i;
-	gboolean val, size_state;
-	const gchar **paths;
-
-	paths = size_paths;
-
-	val = gtk_gs_get_override_size(GTK_GS(ps_view->priv->gs));
-	bonobo_ui_component_set_prop(ps_view->priv->uic, paths[0],
-								 "state", val?"1":"0", NULL);
-	for(i = 1; paths[i] != NULL; i++) {
-		bonobo_ui_component_set_prop(ps_view->priv->uic, paths[i],
-									 "sensitive", val?"1":"0", NULL);
-		size_state = i - 1 == gtk_gs_get_default_size(GTK_GS(ps_view->priv->gs));
-		bonobo_ui_component_set_prop(ps_view->priv->uic, paths[i],
-									 "state", size_state?"1":"0", NULL);
-	}
-}
-
 BonoboUIVerb ggv_postscript_view_verbs[] = {
 	BONOBO_UI_VERB("SettingsPreferences", verb_SettingsPreferences),
 	BONOBO_UI_VERB("FileSaveMarked", verb_FileSaveMarked),
@@ -973,6 +992,7 @@ ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 	g_return_if_fail(ps_view != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
 
+	bonobo_ui_component_freeze(ps_view->priv->uic, NULL);
 	/* Set up the UI from an XML file. */
 	bonobo_ui_util_set_ui(ps_view->priv->uic, DATADIR,
 						  "ggv-postscript-view-ui.xml", 
@@ -981,25 +1001,24 @@ ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 								 "placement", ggv_right_panel?"right":"left", NULL);
 	bonobo_ui_component_set_prop(ps_view->priv->uic, "/Sidebar",
 								 "hidden", ggv_panel?"0":"1", NULL);
-	bonobo_ui_component_object_set(ps_view->priv->uic, "/Sidebar/GgvSidebar",
-								   BONOBO_OBJREF(ps_view->priv->sidebar), NULL);
 	bonobo_ui_component_add_verb_list_with_data(ps_view->priv->uic,
 												ggv_postscript_view_verbs,
 												ps_view);
+	bonobo_ui_component_thaw(ps_view->priv->uic, NULL);
+#if 1
+	bonobo_ui_component_object_set(ps_view->priv->uic, "/Sidebar/GgvSidebar",
+								   BONOBO_OBJREF(ps_view->priv->sidebar), NULL);
+#endif
 
-	for(i = 1; size_paths[i]; i++) 
+	for(i = 0; size_paths[i]; i++) 
 		bonobo_ui_component_add_listener(ps_view->priv->uic,
 										 size_paths[i] + strlen("/commands/"),
 										 listener_Size_cb, ps_view);
 
-	for(i = 1; orientation_paths[i]; i++) 
+	for(i = 0; orientation_paths[i]; i++) 
 		bonobo_ui_component_add_listener(ps_view->priv->uic,
 										 orientation_paths[i] + strlen("/commands/"),
 										 listener_Orientation_cb, ps_view);
-	bonobo_ui_component_add_listener(ps_view->priv->uic, "OverrideSize",
-									 listener_Toggle_cb, ps_view);
-	bonobo_ui_component_add_listener(ps_view->priv->uic, "OverrideOrientation",
-									 listener_Toggle_cb, ps_view);
 	set_file_items_sensitivity(ps_view,
 							   ggv_postscript_view_get_page_count(ps_view) > 0);
 	set_page_items_sensitivity(ps_view);
@@ -1268,18 +1287,15 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 
 		gtk_gs_set_default_orientation(GTK_GS(ps_view->priv->gs), orient);
 		notify_orientation_change(ps_view);
-		/*
 		sync_orientation_items(ps_view);
-		*/
-
 		break;
 	}
 	case PROP_OVERRIDE_ORIENTATION: {
-		 gtk_gs_set_override_orientation(GTK_GS(ps_view->priv->gs),
-										 *(CORBA_boolean *)arg->_value);
-		 notify_orientation_change(ps_view);
-		 sync_orientation_items(ps_view);
-		 break;
+		gtk_gs_set_override_orientation(GTK_GS(ps_view->priv->gs),
+										*(CORBA_boolean *)arg->_value);
+		notify_orientation_change(ps_view);
+		sync_orientation_items(ps_view);
+		break;
 	}
 	case PROP_DEFAULT_SIZE: {
 		gint size;
@@ -1290,18 +1306,16 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 		size_name = *(GNOME_GGV_Size *)arg->_value;
 
 		size = gtk_gs_get_size_index(size_name,
-									   gtk_gs_defaults_get_paper_sizes());
-
+									 gtk_gs_defaults_get_paper_sizes());
 		gtk_gs_set_default_size(GTK_GS(ps_view->priv->gs), size);
 		sync_size_items(ps_view);
-
 		break;
 	}
 	case PROP_OVERRIDE_SIZE: {
-		 gtk_gs_set_override_size(GTK_GS(ps_view->priv->gs),
-								   *(CORBA_boolean *)arg->_value);
-		 sync_size_items(ps_view);
-		 break;
+		gtk_gs_set_override_size(GTK_GS(ps_view->priv->gs),
+								 *(CORBA_boolean *)arg->_value);
+		sync_size_items(ps_view);
+		break;
 	}
 	case PROP_ANTIALIASING: {
 		 gtk_gs_set_override_size(GTK_GS(ps_view->priv->gs),
@@ -1544,6 +1558,7 @@ GNOME_GGV_Orientation
 ggv_postscript_view_get_default_orientation(GgvPostScriptView *ps_view)
 {
 	BonoboArg *arg;
+	GNOME_GGV_Orientation orient;
 
 	g_return_val_if_fail(ps_view != NULL, 0);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), 0);
@@ -1554,7 +1569,9 @@ ggv_postscript_view_get_default_orientation(GgvPostScriptView *ps_view)
 		return 0;
 
 	g_assert(arg->_type == TC_GNOME_GGV_Orientation);
-	return *(GNOME_GGV_Orientation *) arg->_value;
+	orient = *(GNOME_GGV_Orientation *) arg->_value;
+	bonobo_arg_release(arg);
+	return orient;
 }
 
 static void
@@ -1592,6 +1609,7 @@ ggv_postscript_view_destroy(BonoboObject *object)
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->sidebar));
 		ps_view->priv->sidebar = NULL;
 	}
+	ps_view_clean_tmp_file(ps_view);
 
 	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
@@ -1607,6 +1625,13 @@ ggv_postscript_view_finalize(GObject *object)
 
 	ps_view = GGV_POSTSCRIPT_VIEW(object);
 
+	ps_view_clean_tmp_file(ps_view);
+
+	if(ps_view->priv->save_path) {
+		g_free(ps_view->priv->save_path);
+		ps_view->priv->save_path = NULL;
+	}
+
 	g_free(ps_view->priv);
 
 	G_OBJECT_CLASS(ggv_postscript_view_parent_class)->finalize(object);
@@ -1661,6 +1686,22 @@ impl_GNOME_GGV_PostScriptView_reload(PortableServer_Servant servant,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(bonobo_object_from_servant(servant));
 	gtk_gs_reload(GTK_GS(ps_view->priv->gs));
+	notify_page_count_change(ps_view);
+	notify_page_change(ps_view);
+	notify_status_change(ps_view);
+}
+
+static void
+impl_GNOME_GGV_PostScriptView_close(PortableServer_Servant servant,
+									CORBA_Environment *ev)
+{
+	GgvPostScriptView *ps_view;
+
+	ps_view = GGV_POSTSCRIPT_VIEW(bonobo_object_from_servant(servant));
+	gtk_gs_load(GTK_GS(ps_view->priv->gs), NULL);
+	notify_page_count_change(ps_view);
+	notify_page_change(ps_view);
+	notify_status_change(ps_view);
 }
 
 static void
@@ -1682,6 +1723,7 @@ ggv_postscript_view_class_init(GgvPostScriptViewClass *klass)
 	epv->getDocument = impl_GNOME_GGV_PostScriptView_getDocument;
 	epv->getPages = impl_GNOME_GGV_PostScriptView_getPages;
 	epv->reload = impl_GNOME_GGV_PostScriptView_reload;
+	epv->close = impl_GNOME_GGV_PostScriptView_close;
 
 	/* this seems as a nice place to load the prefs */
 	ggv_prefs_load();
@@ -1756,7 +1798,12 @@ property_control_get_cb(BonoboPropertyControl *property_control,
 static void
 document_changed_cb(GtkGS *gs, gpointer data)
 {
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+
 	gtk_gs_reload(gs);
+	notify_page_count_change(GGV_POSTSCRIPT_VIEW(data));
+	notify_page_change(GGV_POSTSCRIPT_VIEW(data));
+	notify_status_change(GGV_POSTSCRIPT_VIEW(data));
 }
 
 static void
@@ -1764,8 +1811,11 @@ interpreter_message_cb(GtkGS *gs, gchar *msg, gpointer data)
 {
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 
+#if 0
+	/* actually, these are quite useless, so we won't show them... */
 	if(ps_view->priv->msg_win)
 		ggv_msg_window_add_text(ps_view->priv->msg_win, msg, TRUE);
+#endif
 
 #ifdef BONOBO_DEBUG
 	g_warning(msg);
@@ -1778,6 +1828,7 @@ interpreter_error_cb(GtkGS *gs, gint status, gpointer data)
 	notify_title_change(GGV_POSTSCRIPT_VIEW(data));
 	notify_page_change(GGV_POSTSCRIPT_VIEW(data));
 	notify_page_count_change(GGV_POSTSCRIPT_VIEW(data));
+	notify_status_change(GGV_POSTSCRIPT_VIEW(data));
 }
 
 static void
@@ -1787,7 +1838,7 @@ view_realized_cb(GtkGS *gs, gpointer data)
 
 	cursor = cursor_get(GTK_WIDGET(gs)->window, CURSOR_HAND_OPEN);
 	gdk_window_set_cursor(GTK_WIDGET(gs)->window, cursor);
-	gdk_cursor_destroy(cursor);
+	gdk_cursor_unref(cursor);
 }
 
 static void
@@ -1798,44 +1849,82 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 	GtkGS *gs = GTK_GS(ps_view->priv->gs);
 
 	if(!strcmp(entry->key, "/apps/ggv/gtkgs/respect_eof")) 
-		gtk_gs_set_respect_eof(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/respect_eof", NULL));
+		gtk_gs_set_respect_eof(gs, gconf_value_get_bool(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/override_orientation")) {
-		gtk_gs_set_override_orientation(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/override_orientation",
-																  NULL));
+		gtk_gs_set_override_orientation(gs, 
+										gconf_value_get_bool(entry->value));
 		sync_orientation_items(ps_view);
 	}
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/orientation")) {
-		gtk_gs_set_default_orientation(gs, gconf_client_get_int(client, "/apps/ggv/gtkgs/orientation",
-																NULL));
+		gtk_gs_set_default_orientation(gs, gconf_value_get_int(entry->value));
 		sync_orientation_items(ps_view);
 	}
 #if 0
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/zoom")) {
-		gtk_gs_set_zoom(gs, gconf_client_get_float(client, "/apps/ggv/gtkgs/zoom",
-												   NULL));
+		gtk_gs_set_zoom(gs, gconf_value_get_float(entry->value));
 	}
 #endif
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/size")) {
-		gtk_gs_set_default_size(gs, gconf_client_get_int(client, "/apps/ggv/gtkgs/size",
-														 NULL));
+		gtk_gs_set_default_size(gs, gconf_value_get_int(entry->value));
 		sync_size_items(ps_view);
 	}
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/antialiasing"))
-		gtk_gs_set_antialiasing(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/antialiasing",
-														  NULL));
+		gtk_gs_set_antialiasing(gs, gconf_value_get_bool(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/override_size")) {
-		gtk_gs_set_override_size(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/override_size",
-														   NULL));
+		gtk_gs_set_override_size(gs, gconf_value_get_bool(entry->value));
 		sync_size_items(ps_view);
 	}
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/watch_doc"))
-		gtk_gs_set_watch_file(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/watch_doc",
-														NULL));
+		gtk_gs_set_watch_file(gs, gconf_value_get_bool(entry->value));
+	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/scrollstep"))
+		gtk_gs_set_scroll_step(gs, gconf_value_get_float(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/show_scroll_rect"))
-		gtk_gs_set_show_scroll_rect(gs, gconf_client_get_bool(client, "/apps/ggv/gtkgs/show_scroll_rect",
-															  NULL));
+		gtk_gs_set_show_scroll_rect(gs, gconf_value_get_bool(entry->value));
+	else if(!strcmp(entry->key, "/apps/ggv/control/autojump"))
+	   ps_view->priv->pane_auto_jump = gconf_value_get_bool(entry->value);
+	else if(!strcmp(entry->key, "/apps/ggv/control/pageflip"))
+	   ps_view->priv->page_flip = gconf_value_get_bool(entry->value);
 }
 
+static gboolean
+sidebar_key_press_event(GtkWidget *widget, GdkEventKey *event,
+						gpointer data)
+{
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+	GtkGS *gs;
+
+	if(ps_view->priv->gs)
+		gs = GTK_GS(ps_view->priv->gs);
+	else
+		return TRUE;
+
+	switch(event->keyval) {
+	case GDK_Left:
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
+		break;
+	case GDK_Right:
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
+		break;
+	case GDK_Up:
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
+		break;
+	case GDK_Down:
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, TRUE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
+		break;
+	default:
+		return FALSE;
+		break;
+	}
+	return TRUE;
+}
 
 GgvPostScriptView *
 ggv_postscript_view_construct(GgvPostScriptView *ps_view,
@@ -1854,7 +1943,7 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 
 	ps_view->priv->gconf_notify_id =
 		gconf_client_notify_add(gtk_gs_defaults_gconf_client(),
-								"/apps/ggv/gtkgs",
+								"/apps/ggv",
 								(GConfClientNotifyFunc)ggv_postscript_view_prefs_changed,
 								ps_view, NULL, NULL);
 															 
@@ -1960,7 +2049,17 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 	ps_view->priv->uic = bonobo_ui_component_new("GgvPostScriptView");
 
 	ps_view->priv->sidebar = ggv_sidebar_new(ps_view);
+	g_signal_connect(G_OBJECT(ggv_sidebar_get_checklist(ps_view->priv->sidebar)),
+					 "key_press_event", G_CALLBACK(sidebar_key_press_event), ps_view);
 
+	ps_view->priv->pane_auto_jump =
+		gconf_client_get_bool(gtk_gs_defaults_gconf_client(),
+							  "/apps/ggv/control/autojump",
+							  NULL);
+	ps_view->priv->page_flip =
+		gconf_client_get_bool(gtk_gs_defaults_gconf_client(),
+							  "/apps/ggv/control/pageflip",
+							  NULL);
 	return ps_view;
 }
 
@@ -1996,3 +2095,15 @@ ggv_postscript_view_set_popup_ui_component(GgvPostScriptView *ps_view,
 	ps_view->priv->popup_uic = uic;
 	_set_page_items_sensitivity(ps_view, uic);
 }
+
+gboolean
+ggv_postscript_view_get_auto_jump(GgvPostScriptView *ps_view)
+{
+	return ps_view->priv->pane_auto_jump;
+}
+
+gboolean
+ggv_postscript_view_get_page_flip(GgvPostScriptView *ps_view)
+{
+	return ps_view->priv->page_flip;
+}
