@@ -296,35 +296,6 @@ right_panel_clicked(GtkWidget *w, gpointer user_data)
                               ggv_right_panel, NULL);
 }
 
-#if 0
-static void
-ggv_prefs_dialog_ok_clicked(GtkWidget *widget, gpointer *data)
-{
-        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(data);
-
-        gtk_widget_hide(GTK_WIDGET(dlg));
-        ggv_prefs_dialog_apply(GGV_PREFS_DIALOG(dlg));
-}
-
-
-static void
-ggv_prefs_dialog_apply_clicked(GtkWidget *widget, gpointer *data)
-{
-        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(data);
-
-        ggv_prefs_dialog_apply(GGV_PREFS_DIALOG(dlg));
-}
-
-
-static void
-ggv_prefs_dialog_cancel_clicked(GtkWidget *widget, gpointer *data)
-{
-        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(data);
-
-        gtk_widget_hide(GTK_WIDGET(dlg));
-}
-#endif /* 0 */
-
 static gboolean
 ggv_prefs_dialog_delete_event(GtkWidget *widget, GdkEventAny *event)
 {
@@ -349,20 +320,6 @@ ggv_prefs_dialog_init (GgvPrefsDialog *dlg)
 {
 }
 
-static void
-ggv_prefs_dialog_changed(GgvPrefsDialog *dlg)
-{
-        gtk_widget_set_sensitive(dlg->ok, TRUE);
-        gtk_widget_set_sensitive(dlg->apply, TRUE);
-}
-
-static void
-prefs_dlg_change_handler(GtkWidget *widget, gpointer data)
-{
-        GgvPrefsDialog *dlg = GGV_PREFS_DIALOG(data);
-        ggv_prefs_dialog_changed(dlg);
-}
-
 static void
 ggv_prefs_dialog_setup(GgvPrefsDialog *dlg)
 {
@@ -465,11 +422,6 @@ ggv_prefs_dialog_setup(GgvPrefsDialog *dlg)
         gtk_spin_button_set_value(GTK_SPIN_BUTTON(dlg->scroll_step),
                                   gtk_gs_defaults_get_scroll_step());
 	gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_MOUSE);
-
-        gtk_widget_grab_default(dlg->cancel);
-
-        gtk_widget_set_sensitive(dlg->ok, FALSE);
-        gtk_widget_set_sensitive(dlg->apply, FALSE);
 }
 
 void
