@@ -13,9 +13,15 @@
 #include <gconf/gconf-client.h>
 #include <errno.h>
 #include <signal.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <stdio.h>
 
 #ifdef __cplusplus
 extern "C" {
+#if 0
+} /* let's fool emacs */
+#endif /* 0 */
 #endif /* __cplusplus */
         
 #define GTK_GS(obj)         GTK_CHECK_CAST (obj, gtk_gs_get_type (), GtkGS)
@@ -37,6 +43,13 @@ typedef enum
         GTK_GS_ORIENTATION_LANDSCAPE = 1,
 } GtkGSOrientation;
 
+typedef enum
+{
+        GTK_GS_ZOOM_ABSOLUTE = 0,
+        GTK_GS_ZOOM_FIT_WIDTH = 1,
+        GTK_GS_ZOOM_FIT_PAGE = 2
+} GtkGSZoomMode;
+
 struct _GtkGS
 {
         GtkWidget widget;                    /* the main widget */
@@ -50,16 +63,13 @@ struct _GtkGS
         gint scroll_width, scroll_height;
         gboolean show_scroll_rect;
 
+        GtkGSZoomMode zoom_mode;
+
 	GdkPixmap *bpixmap;			/* Backing pixmap */
 	int        use_bpixmap;
 
         long       message_window;  /* Used by ghostview to receive messages from app */
         
-#if 0
-	GtkAdjustment* hadj;
-	GtkAdjustment* vadj;
-#endif
-
 	int        disable_start;		/* Can the interpreter be started? */
 	pid_t      interpreter_pid;		/* PID of interpreter, -1 if none  */
 	int        interpreter_input;         /* stdin of interpreter            */
@@ -82,6 +92,7 @@ struct _GtkGS
 	gboolean    busy;			/* Is gs busy drawing? */
 	gboolean    changed;			/* Anything changed since setup */
         gfloat      zoom_factor;
+        gfloat      scroll_step;
         gint        current_page;
         gboolean    structured_doc;
         gboolean    loaded;
@@ -113,9 +124,10 @@ struct _GtkGS
         gboolean    override_orientation;
         gint        fallback_orientation;  /* Orientation to use if override */
         gint        real_orientation;      /* Real orientation from the document */
-        gint       *pages_marked;
 
-        guint      gconf_notify_id;
+        const gchar *gs_status;            /* GtkGS status */
+
+        guint avail_w, avail_h;
 };
 
 struct _GtkGSClass
@@ -131,7 +143,9 @@ struct _GtkGSClass
         GConfClient *gconf_client;
 
         void (*interpreter_message)(GtkGS *, gchar *, gpointer);
+        void (*interpreter_error)(GtkGS *, gint, gpointer);
         void (*document_changed)(GtkGS *, gpointer);
+        void (*set_scroll_adjustments)(GtkGS *, GtkAdjustment *, GtkAdjustment *);
 };
 
 
@@ -157,6 +171,7 @@ GType      gtk_gs_get_type (void);
 GtkWidget *gtk_gs_new_from_file(GtkAdjustment *hadj, GtkAdjustment *vadj, gchar *fname);
 GtkWidget *gtk_gs_new(GtkAdjustment *hadj, GtkAdjustment *vadj);
 gboolean   gtk_gs_load (GtkGS *gs, const gchar *fname);
+void       gtk_gs_reload(GtkGS *gs);
 
 /* control functions */
 void         gtk_gs_center_page (GtkGS *gs);
@@ -178,6 +193,8 @@ void         gtk_gs_set_default_size(GtkGS *gs, gint size);
 gint         gtk_gs_get_default_size(GtkGS *gs);
 void         gtk_gs_set_zoom(GtkGS *gs, gfloat zoom);
 gfloat       gtk_gs_get_zoom(GtkGS *gs);
+void         gtk_gs_set_scroll_step(GtkGS *gs, gfloat scroll_step);
+gfloat       gtk_gs_get_scroll_step(GtkGS *gs);
 gfloat       gtk_gs_zoom_to_fit(GtkGS *gs, gboolean fit_width);
 void         gtk_gs_set_center(GtkGS *gs, gfloat hval, gfloat vval);
 gint         gtk_gs_get_orientation(GtkGS *gs);
@@ -194,21 +211,28 @@ gboolean     gtk_gs_get_override_size(GtkGS *gs);
 const gchar *gtk_gs_get_document_title(GtkGS *widget);
 guint        gtk_gs_get_document_numpages(GtkGS *widget);
 const gchar *gtk_gs_get_document_page_label(GtkGS *widget, int page);
-gint         gtk_gs_count_marked_pages(GtkGS *widget);
 void         gtk_gs_set_show_scroll_rect(GtkGS *gs, gboolean f);
 gboolean     gtk_gs_get_show_scroll_rect(GtkGS *gs);
 
 void         gtk_gs_start_scroll(GtkGS *gs);
 void         gtk_gs_end_scroll(GtkGS *gs);
 
-/* utility functions */
-gint     gtk_gs_get_size_index(gchar *string, GtkGSPaperSize *size);
+void          gtk_gs_set_zoom_mode(GtkGS *gs, GtkGSZoomMode zoom_mode);
+GtkGSZoomMode gtk_gs_get_zoom_mode(GtkGS *gs);
 
-#define GTK_GS_IS_COMPRESSED(gs)       (gs->gs_filename_unc != NULL)
+void         gtk_gs_set_available_size(GtkGS *gs, guint avail_w, guint avail_h);
+
+/* utility functions */
+gint     gtk_gs_get_size_index(const gchar *string, GtkGSPaperSize *size);
+
+gchar   *gtk_gs_get_postscript(GtkGS *gs, gint *pages);
+
+#define GTK_GS_IS_COMPRESSED(gs)       (GTK_GS(gs)->gs_filename_unc != NULL)
 #define GTK_GS_GET_PS_FILE(gs)         (GTK_GS_IS_COMPRESSED(gs) ? \
-                                        gs->gs_filename_unc : \
-                                        gs->gs_filename)
-#define GTK_GS_IS_PDF(gs)              (gs->gs_filename_dsc != NULL)
+                                        GTK_GS(gs)->gs_filename_unc : \
+                                        GTK_GS(gs)->gs_filename)
+#define GTK_GS_IS_PDF(gs)              (GTK_GS(gs)->gs_filename_dsc != NULL)
+#define GTK_GS_IS_STRUCTURED_DOC(gs)   (GTK_GS(gs)->structured_doc)
 
 #ifdef __cplusplus
 }
